module Fastlane
  module Actions
    class GetGradleDataAction < Action
      def self.run(params)
        gradle_file_path = Helper::GradleFileFinderHelper.find_file(params)

        begin
          gradle_data = Helper::GradleParserHelper.parse(gradle_file_path)
          Actions.lane_context[SharedValues::GM_RAW_DATA] = gradle_data
          UI.success('👍 gradle data successfully parsed')
          return gradle_data
        rescue => err
          UI.error("An exception occurred while parsing the gradle file: #{err}")
        end
      end

      def self.description
        'Get the parsed Gradle file of an Android project.'
      end

      def self.authors
        ['Helder Pinhal']
      end

      def self.return_value
        'Returns the parsed Gradle file.'
      end

      def self.details
        [
            'Get the parsed Gradle file of an Android project.',
            'This action will return the parsed Gradle file based on the path you\'ve specified.'
        ].join('\n')
      end

      def self.available_options
        [
            FastlaneCore::ConfigItem.new(key: :module_name,
                                         env_name: 'GRADLE_MANAGER_MODULE_NAME',
                                         description: 'The name of the application source folder in the Android project (default: app)',
                                         optional: true,
                                         type: String,
                                         default_value: 'app'),
            FastlaneCore::ConfigItem.new(key: :gradle_file_path,
                                         env_name: 'GRADLE_MANAGER_GRADLE_FILE_PATH',
                                         description: 'The relative path to the gradle file (default:app/build.gradle)',
                                         optional: true,
                                         type: String,
                                         default_value: nil)
        ]
      end

      def self.is_supported?(platform)
        [:android].include?(platform)
      end
    end
  end
end
