module Fastlane
  module Actions
    class GetVersionNameAction < Action
      def self.run(params)
        flavor ||= params[:flavor_name]

        begin
          gradle_data ||= params[:cached_data]

          if gradle_data.nil?
            gradle_file_path = Helper::GradleFileFinderHelper.find_file(params)
            gradle_data = Helper::GradleParserHelper.parse(gradle_file_path)
          end

          return Helper::GradleDataFinderHelper.find_property(gradle_data, :version_name, flavor)
        rescue => err
          UI.error("An exception occurred while parsing the gradle file: #{err}")
        end
      end

      def self.description
        'Get the parsed Gradle file of an Android project.'
      end

      def self.authors
        ['Helder Pinhal']
      end

      def self.return_value
        'Returns the parsed Gradle file.'
      end

      def self.details
        [
          'Get the parsed Gradle file of an Android project.',
          'This action will return the parsed Gradle file based on the path you\'ve specified.'
        ].join('\n')
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :cached_data,
                                       env_name: 'GRADLE_MANAGER_CACHED_DATA',
                                       description: 'The cached data from get_gradle_data. If not supplied then the action will try to fetch it',
                                       optional: true,
                                       type: Hash,
                                       default_value: nil),
          FastlaneCore::ConfigItem.new(key: :module_name,
                                       env_name: 'GRADLE_MANAGER_MODULE_NAME',
                                       description: 'The name of the application source folder in the Android project (default: app)',
                                       optional: true,
                                       type: String,
                                       default_value: 'app'),
          FastlaneCore::ConfigItem.new(key: :gradle_file_path,
                                       env_name: 'GRADLE_MANAGER_GRADLE_FILE_PATH',
                                       description: 'The relative path to the gradle file (default:app/build.gradle)',
                                       optional: true,
                                       type: String,
                                       default_value: nil),
          FastlaneCore::ConfigItem.new(key: :flavor_name,
                                       env_name: 'GRADLE_MANAGER_FLAVOR_NAME',
                                       description: 'The name of the product flavor',
                                       optional: true,
                                       type: String,
                                       default_value: nil)
        ]
      end

      def self.is_supported?(platform)
        [:android].include?(platform)
      end
    end
  end
end
